// Copyright (C) 2017 by Pedro Mendes, Virginia Tech Intellectual
// Properties, Inc., University of Heidelberg, and University of
// of Connecticut School of Medicine.
// All rights reserved.

// Copyright (C) 2010 - 2016 by Pedro Mendes, Virginia Tech Intellectual
// Properties, Inc., University of Heidelberg, and The University
// of Manchester.
// All rights reserved.

// Copyright (C) 2008 - 2009 by Pedro Mendes, Virginia Tech Intellectual
// Properties, Inc., EML Research, gGmbH, University of Heidelberg,
// and The University of Manchester.
// All rights reserved.

#include "test000063.h"

#include <sstream>
#include "utilities.hpp"
#include "copasi/CopasiDataModel/CDataModel.h"
#include "copasi/model/CModel.h"
#include "copasi/model/CModelValue.h"
#include "copasi/function/CFunctionDB.h"

#include "sbml/SBMLDocument.h"
#include "sbml/Model.h"
#include "sbml/Rule.h"
#include "sbml/Species.h"
#include "sbml/Parameter.h"
#include "sbml/math/ASTNode.h"

#include "copasi/core/CRootContainer.h"

CDataModel* test000063::pCOPASIDATAMODEL = NULL;

void test000063::setUp()
{
  // Create the root container.
  CRootContainer::init(0, NULL, false);
  // Create the global data model.
  pCOPASIDATAMODEL = CRootContainer::addDatamodel();
}

void test000063::tearDown()
{
  CRootContainer::destroy();
}

void test000063::test_events_1()
{
  CDataModel* pDataModel = pCOPASIDATAMODEL;
  CPPUNIT_ASSERT(pDataModel->importSBMLFromString(test000063::MODEL_STRING1));
  CModel* pModel = pDataModel->getModel();
  CPPUNIT_ASSERT(pModel != NULL);
  CPPUNIT_ASSERT(pModel->getCompartments().size() == 1);
  CPPUNIT_ASSERT(pModel->getMetabolites().size() == 2);
  CPPUNIT_ASSERT(pModel->getReactions().size() == 0);
  CPPUNIT_ASSERT(pModel->getEvents().size() == 1);
  // run a simulation for 4 seconds and check the values of both species at 1
  // second and at 3 seconds
}

/**
 * Simple model with two species. One is determined by an assignment, the
 * other is set in an event assignment.
 */
const char* test000063::MODEL_STRING1 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfRules>"
  "     <assignmentRule variable=\"species_1\">"
  "      <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "       <apply>"
  "         <times/>"
  "         <ci>species_2</ci>"
  "         <cn> 2.0 </cn>"
  "       </apply>"
  "      </math>"
  "     </assignmentRule>"
  "    </listOfRules>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 2.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <cn> 2.0 </cn>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species. One is determined by an ode , the
 * other is set in an event assignment.
 */
const char* test000063::MODEL_STRING2 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfRules>"
  "     <rateRule variable=\"species_1\">"
  "      <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "       <cn> 2.0 </cn>"
  "      </math>"
  "     </rateRule>"
  "    </listOfRules>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci>species_1</ci>"
  "              <cn> 8.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <cn> 2.0 </cn>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species. The compartment volume is determined by a
 * rate rule.
 * The second species is set by an event that is dependent on the compartment
 * volume.
 */
const char* test000063::MODEL_STRING3 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\" constant=\"false\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfRules>"
  "     <rateRule variable=\"compartment_1\">"
  "      <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "       <cn> 2.0 </cn>"
  "      </math>"
  "     </rateRule>"
  "    </listOfRules>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> compartment_1 </ci>"
  "              <cn> 6.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <cn> 2.0 </cn>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species and a parameter. The parameter is determined
 * by an ode rule.
 * The second species is set by an event that is dependent on the parameter
 * volume.
 */
const char* test000063::MODEL_STRING4 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\" />"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfParameters>"
  "     <parameter id=\"parameter_1\" name=\"parameter_1\" value=\"2.0\" constant=\"false\" units=\"dimensionless\"/>"
  "    </listOfParameters>"
  "    <listOfRules>"
  "     <rateRule variable=\"parameter_1\">"
  "      <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "       <cn> 2.0 </cn>"
  "      </math>"
  "     </rateRule>"
  "    </listOfRules>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> parameter_1 </ci>"
  "              <cn> 6.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <cn> 2.0 </cn>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species and two reactions.
 * The second species is set by an event that is dependent on the reaction
 * rate of the second reaction.
 */
const char* test000063::MODEL_STRING5 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\" />"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfReactions>"
  "     <reaction id=\"reaction_1\" name=\"constant flux\">"
  " <listOfProducts>"
  " < speciesReference species = \"species_1\"/>"
  "       </listOfProducts>"
  "       <kineticLaw>"
  "        <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "          <cn> 2.0 </cn>"
  "        </math>"
  "       </kineticLaw>"
  "     </reaction>"
  "     <reaction id=\"reaction_2\" name=\"constant flux\">"
  " <listOfReactants>"
  " < speciesReference species = \"species_1\"/>"
  "       </listOfReactants>"
  "       <listOfProducts>"
  "        <speciesReference species=\"species_2\"/>"
  "       </listOfProducts>"
  "       <kineticLaw>"
  "        <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "          <ci> species_1 </ci>"
  "        </math>"
  "       </kineticLaw>"
  "     </reaction>"
  "    </listOfReactions>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> reaction_2 </ci>"
  "              <cn> 6.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <cn> 2.0 </cn>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species and two reactions.
 * The second species is set by an event that is dependent on the reaction
 * rate of the second reaction.
 * The second species is also set by a second time dependent event.
 */
const char* test000063::MODEL_STRING6 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\" />"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfReactions>"
  "     <reaction id=\"reaction_1\" name=\"constant flux\">"
  " <listOfProducts>"
  " < speciesReference species = \"species_1\"/>"
  "       </listOfProducts>"
  "       <kineticLaw>"
  "        <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "          <cn> 2.0 </cn>"
  "        </math>"
  "       </kineticLaw>"
  "     </reaction>"
  "     <reaction id=\"reaction_2\" name=\"constant flux\">"
  " <listOfReactants>"
  " < speciesReference species = \"species_1\"/>"
  "       </listOfReactants>"
  "       <listOfProducts>"
  "        <speciesReference species=\"species_2\"/>"
  "       </listOfProducts>"
  "       <kineticLaw>"
  "        <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "          <ci> species_1 </ci>"
  "        </math>"
  "       </kineticLaw>"
  "     </reaction>"
  "    </listOfReactions>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> reaction_2 </ci>"
  "              <cn> 6.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <cn> 2.0 </cn>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "      <event id=\"event_2\" name=\"event_2\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 8.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <cn> 2.0 </cn>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species. One species is determined by a rate rule,
 * the other is set by an event that is determined by the first species.
 * The assignment expression references the species itself.
 */
const char* test000063::MODEL_STRING7 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\" />"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfRules>"
  "     <rateRule variable=\"species_1\">"
  "      <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "       <cn> 2.0 </cn>"
  "      </math>"
  "     </rateRule>"
  "    </listOfRules>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> species_1 </ci>"
  "              <cn> 6.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species.
 * Both species are set by event assignments.
 * The event is supposed to fire every three seconds from second 2.
 * This is achieved by simulating the modulo function.
 * This model is solely made of events.
 */
const char* test000063::MODEL_STRING8 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <lt/>"
  "              <apply>"
  "                <minus/>"
  "                 <apply>"
  "                   <divide/>"
  "                   <apply>"
  "                     <minus/>"
  "                     <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "                     <cn> 2.0 </cn>"
  "                   </apply>"
  "                   <cn> 3.0 </cn>"
  "                 </apply>"
  "                 <apply>"
  "                   <floor/>"
  " <apply>"
  " < divide / > "
  " <apply>"
  " < minus / > "
  " < csymbol encoding = \"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "                       <cn> 2.0 </cn>"
  "                     </apply>"
  "                     <cn> 3.0 </cn>"
  "                   </apply>"
  "                 </apply>"
  " < / apply > "
  " <cn> 0.001 < / cn > "
  " < / apply > "
  " < / math > "
  " < / trigger > "
  " <listOfEventAssignments>"
  " < eventAssignment variable = \"species_1\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_1 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species.
 * This model represents an event endlessloop.
 * There are three events, the first one triggers when time is greater 1
 * second and it modifies species_1.
 * The second event is triggered if species_1 becomes larger than species_2 and
 * it enlarges species_2.
 * And the third triggers when species_2 becomes larger than species_1 and
 * enlarger species_1.
 */
const char* test000063::MODEL_STRING9 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_1\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "      <event id=\"event_2\" name=\"event_2\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> species_2 </ci>"
  "              <ci> species_1 </ci>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_1\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "      <event id=\"event_3\" name=\"event_3\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> species_1 </ci>"
  "              <ci> species_2 < / ci >"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_1 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species.
 * Two events trigger at the same time.
 * Each modifies one of the species and references the other species.
 */
const char* test000063::MODEL_STRING10 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_1\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "      <event id=\"event_2\" name=\"event_2\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_1 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species.
 * Two events. One triggers at a certain time and modifies one species. This
 * modification triggers the second event at the same time.
 * Although both events trigger at the same point in time, the result should be
 * different from the model above.
 */
const char* test000063::MODEL_STRING11 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_1\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "      <event id=\"event_2\" name=\"event_2\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> species_1 </ci>"
  "              <cn> 7.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_1 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species. One is determined by an assignment, the
 * other is set in an event assignment.
 * The event has a delay.
 */
const char* test000063::MODEL_STRING12 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfRules>"
  "     <assignmentRule variable=\"species_1\">"
  "      <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "       <apply>"
  "         <times/>"
  "         <ci>species_2</ci>"
  "         <cn> 2.0 </cn>"
  "       </apply>"
  "      </math>"
  "     </assignmentRule>"
  "    </listOfRules>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 2.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <cn> 2.0 </cn>"
  "          </math>"
  "        </delay>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <cn> 2.0 </cn>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species.
 * Two events trigger at the same time.
 * Each modifies one of the species and references the other species.
 * Both events use the same delay.
 */
const char* test000063::MODEL_STRING13 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <cn> 2.0 </cn>"
  "          </math>"
  "        </delay>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_1\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "      <event id=\"event_2\" name=\"event_2\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <cn> 2.0 </cn>"
  "          </math>"
  "        </delay>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_1 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species.
 * Two events. One triggers at a certain time and modifies one species. This
 * modification triggers the second event at the same time.
 * Although both events trigger at the same point in time, the result should be
 * different from the model above.
 * Both event use the same delay
 */
const char* test000063::MODEL_STRING14 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <cn> 2.0 </cn>"
  "          </math>"
  "        </delay>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_1\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "      <event id=\"event_2\" name=\"event_2\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> species_1 </ci>"
  "              <cn> 7.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <cn> 2.0 </cn>"
  "          </math>"
  "        </delay>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_1 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species.
 * Two events trigger at the same time.
 * Each modifies one of the species and references the other species.
 * Both events use different delays.
 */
const char* test000063::MODEL_STRING15 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <cn> 3.0 </cn>"
  "          </math>"
  "        </delay>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_1\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "      <event id=\"event_2\" name=\"event_2\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <cn> 2.0 </cn>"
  "          </math>"
  "        </delay>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_1 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species.
 * Two events. One triggers at a certain time and modifies one species. This
 * modification triggers the second event at the same time.
 * Although both events trigger at the same point in time, the result should be
 * different from the model above.
 * Both event use different delays.
 */
const char* test000063::MODEL_STRING16 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"3.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 1.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <cn> 3.0 </cn>"
  "          </math>"
  "        </delay>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_1\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_2 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "      <event id=\"event_2\" name=\"event_2\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci> species_1 </ci>"
  "              <cn> 7.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <cn> 2.0 </cn>"
  "          </math>"
  "        </delay>"
  "        <listOfEventAssignments>"
  "          <eventAssignment variable=\"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <apply>"
  "                <times/>"
  "                <cn> 2.0 </cn>"
  "                <ci> species_1 </ci>"
  "              </apply>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;

/**
 * Simple model with two species. One is determined by an ode , the
 * other is set in an event assignment.
 * The delay in the event has to be calculated from the trigger time.
 */
const char* test000063::MODEL_STRING17 =
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
  "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">"
  "  <model id=\"Model_1\" name=\"Model 1\">"
  "    <listOfUnitDefinitions>"
  "      <unitDefinition id=\"volume\">"
  "        <listOfUnits>"
  "          <unit kind=\"litre\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "      <unitDefinition id=\"substance\">"
  "        <listOfUnits>"
  "          <unit kind=\"mole\" scale=\"-3\"/>"
  "        </listOfUnits>"
  "      </unitDefinition>"
  "    </listOfUnitDefinitions>"
  "    <listOfCompartments>"
  "      <compartment id=\"compartment_1\" name=\"compartment\" size=\"1\"/>"
  "    </listOfCompartments>"
  "    <listOfSpecies>"
  "      <species id=\"species_1\" name=\"A\" compartment=\"compartment_1\" initialConcentration=\"0.0\"/>"
  "      <species id=\"species_2\" name=\"B\" compartment=\"compartment_1\" initialConcentration=\"5.0\"/>"
  "    </listOfSpecies>"
  "    <listOfRules>"
  "     <rateRule variable=\"species_1\">"
  "      <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "       <cn> 2.0 </cn>"
  "      </math>"
  "     </rateRule>"
  "    </listOfRules>"
  "    <listOfEvents>"
  "      <event id=\"event_1\" name=\"event_1\">"
  "        <trigger>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <gt/>"
  "              <ci>species_1</ci>"
  "              <cn> 8.0 </cn>"
  "            </apply>"
  "          </math>"
  "        </trigger>"
  "        <delay>"
  "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "            <apply>"
  "              <divide/>"
  "              <csymbol encoding=\"text\" definitionURL=\"http://www.sbml.org/sbml/symbols/time\"> time </csymbol>"
  "              <cn> 2.0 </cn>"
  " < / apply > "
  " < / math > "
  " < / delay > "
  " <listOfEventAssignments>"
  " < eventAssignment variable = \"species_2\">"
  "            <math xmlns=\"http://www.w3.org/1998/Math/MathML\">"
  "              <cn> 2.0 </cn>"
  "            </math>"
  "          </eventAssignment>"
  "        </listOfEventAssignments>"
  "      </event>"
  "    </listOfEvents>"
  "  </model>"
  "</sbml>"
  ;
