// Copyright (C) 2017 by Pedro Mendes, Virginia Tech Intellectual
// Properties, Inc., University of Heidelberg, and University of
// of Connecticut School of Medicine.
// All rights reserved.

// Copyright (C) 2010 - 2016 by Pedro Mendes, Virginia Tech Intellectual
// Properties, Inc., University of Heidelberg, and The University
// of Manchester.
// All rights reserved.

// Copyright (C) 2008 - 2009 by Pedro Mendes, Virginia Tech Intellectual
// Properties, Inc., EML Research, gGmbH, University of Heidelberg,
// and The University of Manchester.
// All rights reserved.

#include "test000067.h"

#include <sstream>
#include "utilities.hpp"
#include "copasi/CopasiDataModel/CDataModel.h"
#include "copasi/model/CModel.h"
#include "copasi/model/CModelValue.h"
#include "copasi/function/CFunctionDB.h"

#include "sbml/SBMLDocument.h"
#include "sbml/Model.h"
#include "sbml/Rule.h"
#include "sbml/Species.h"
#include "sbml/Parameter.h"
#include "sbml/math/ASTNode.h"

#include "test000059.h"

#include "copasi/core/CRootContainer.h"

CDataModel* test000067::pCOPASIDATAMODEL = NULL;

void test000067::setUp()
{
  // Create the root container.
  CRootContainer::init(0, NULL, false);
  // Create the global data model.
  pCOPASIDATAMODEL = CRootContainer::addDatamodel();
}

void test000067::tearDown()
{
  CRootContainer::destroy();
}

void test000067::test_bug1060()
{
  CDataModel* pDataModel = pCOPASIDATAMODEL;
  std::istringstream iss(test000067::MODEL_STRING1);
  CPPUNIT_ASSERT(load_cps_model_from_stream(iss, *pDataModel) == true);
  CPPUNIT_ASSERT(pDataModel->getModel() != NULL);
  const SBMLDocument* pDocument = pDataModel->getCurrentSBMLDocument();
  CPPUNIT_ASSERT(pDocument == NULL);
  CPPUNIT_ASSERT(pDataModel->exportSBMLToString(NULL, 2, 3).empty() == false);
  pDocument = pDataModel->getCurrentSBMLDocument();
  const Model* pSBMLModel = pDocument->getModel();
  CPPUNIT_ASSERT(pSBMLModel != NULL);
  CPPUNIT_ASSERT(pSBMLModel->getNumFunctionDefinitions() == 3);
  CPPUNIT_ASSERT(pSBMLModel->getNumCompartments() == 1);
  CPPUNIT_ASSERT(pSBMLModel->getNumSpecies() == 3);
  CPPUNIT_ASSERT(pSBMLModel->getNumReactions() == 8);
  CPPUNIT_ASSERT(pSBMLModel->getNumRules() == 0);
  CPPUNIT_ASSERT(pSBMLModel->getNumParameters() == 0);
  CPPUNIT_ASSERT(test000059::checkIfIdsUnique(pSBMLModel) == true);
  // check if each reaction call the correct kinetic law
  CFunctionDB* pFunDB = CRootContainer::getFunctionList();
  CEvaluationTree* pModifiedMM = pFunDB->findFunction("Modified MM");
  CPPUNIT_ASSERT(pModifiedMM != NULL);
  CEvaluationTree* pModifiedCF = pFunDB->findFunction("modified constant flux");
  CPPUNIT_ASSERT(pModifiedCF != NULL);
  CEvaluationTree* pConstantFlux = pFunDB->findFunction("Constant flux (irreversible)");
  CPPUNIT_ASSERT(pConstantFlux != NULL);
  unsigned int i, iMax = pSBMLModel->getListOfReactions()->size();
  const Reaction* pReaction = NULL;
  const ASTNode* pRoot = NULL;
  const std::map<CDataObject*, SBase*>& copasi2sbmlmap = pDataModel->getCopasi2SBMLMap();
  std::map<CDataObject*, SBase*>::const_iterator mappos;

  for (i = 0; i < iMax; ++i)
    {
      pReaction = pSBMLModel->getReaction(i);
      CPPUNIT_ASSERT(pReaction != NULL);
      CPPUNIT_ASSERT(pReaction->getKineticLaw() != NULL);
      pRoot = pReaction->getKineticLaw()->getMath();
      CPPUNIT_ASSERT(pRoot != NULL);
      CPPUNIT_ASSERT(pRoot->getType() == AST_TIMES);

      if (pRoot->getChild(0)->getType() == AST_NAME && pRoot->getChild(0)->getName() == pSBMLModel->getCompartment(0)->getId())
        {
          pRoot = pRoot->getChild(1);
        }
      else
        {
          pRoot = pRoot->getChild(0);
        }

      if (pReaction->getName() == "y0Prod" || pReaction->getName() == "y1Prod" || pReaction->getName() == "xProdLin")
        {
          // uses modified constant flux
          CPPUNIT_ASSERT(pRoot->getType() == AST_FUNCTION);
          mappos = copasi2sbmlmap.find(pModifiedCF);
          CPPUNIT_ASSERT(mappos != copasi2sbmlmap.end());
          CPPUNIT_ASSERT(mappos->second != NULL);
          CPPUNIT_ASSERT(pRoot->getName() == mappos->second->getId());
        }
      else if (pReaction->getName() == "xDegMM")
        {
          // uses modifed MM
          CPPUNIT_ASSERT(pRoot->getType() == AST_FUNCTION);
          mappos = copasi2sbmlmap.find(pModifiedMM);
          CPPUNIT_ASSERT(mappos != copasi2sbmlmap.end());
          CPPUNIT_ASSERT(mappos->second != NULL);
          CPPUNIT_ASSERT(pRoot->getName() == mappos->second->getId());
        }
      else if (pReaction->getName() == "xProd")
        {
          // constant flux
          CPPUNIT_ASSERT(pRoot->getType() == AST_FUNCTION);
          mappos = copasi2sbmlmap.find(pConstantFlux);
          CPPUNIT_ASSERT(mappos != copasi2sbmlmap.end());
          CPPUNIT_ASSERT(mappos->second != NULL);
          CPPUNIT_ASSERT(pRoot->getName() == mappos->second->getId());
        }
      else
        {
          // must be mass action
          CPPUNIT_ASSERT(pRoot->getType() == AST_TIMES);
        }
    }
}

const char* test000067::MODEL_STRING1 =
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
  "<!-- generated with COPASI 4.4 (Build 26) (http://www.copasi.org) at 2008-08-04 15:57:12 UTC -->\n"
  "<COPASI xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://www.copasi.org/static/schema.xsd\" versionMajor=\"1\" versionMinor=\"0\" versionDevel=\"26\">\n"
  "  <ListOfFunctions>\n"
  "    <Function key=\"Function_6\" name=\"Constant flux (irreversible)\" type=\"PreDefined\" reversible=\"false\">\n"
  "      <Expression>\n"
  "        v\n"
  "      </Expression>\n"
  "      <ListOfParameterDescriptions>\n"
  "        <ParameterDescription key=\"FunctionParameter_49\" name=\"v\" order=\"0\" role=\"constant\"/>\n"
  "      </ListOfParameterDescriptions>\n"
  "    </Function>\n"
  "    <Function key=\"Function_13\" name=\"Mass action (irreversible)\" type=\"MassAction\" reversible=\"false\">\n"
  "      <Expression>\n"
  "        k1*PRODUCT&lt;substrate_i&gt;\n"
  "      </Expression>\n"
  "      <ListOfParameterDescriptions>\n"
  "        <ParameterDescription key=\"FunctionParameter_81\" name=\"k1\" order=\"0\" role=\"constant\"/>\n"
  "        <ParameterDescription key=\"FunctionParameter_79\" name=\"substrate\" order=\"1\" role=\"substrate\"/>\n"
  "      </ListOfParameterDescriptions>\n"
  "    </Function>\n"
  "    <Function key=\"Function_60\" name=\"Objective Function 1\" type=\"Expression\">\n"
  "      <Expression>\n"
  "        \n"
  "      </Expression>\n"
  "    </Function>\n"
  "    <Function key=\"Function_46\" name=\"Objective Function\" type=\"Expression\">\n"
  "      <Expression>\n"
  "        \n"
  "      </Expression>\n"
  "    </Function>\n"
  "    <Function key=\"Function_63\" name=\"modified constant flux\" type=\"UserDefined\" reversible=\"unspecified\">\n"
  "      <Expression>\n"
  "        k*M\n"
  "      </Expression>\n"
  "      <ListOfParameterDescriptions>\n"
  "        <ParameterDescription key=\"FunctionParameter_297\" name=\"k\" order=\"0\" role=\"constant\"/>\n"
  "        <ParameterDescription key=\"FunctionParameter_268\" name=\"M\" order=\"1\" role=\"modifier\"/>\n"
  "      </ListOfParameterDescriptions>\n"
  "    </Function>\n"
  "    <Function key=\"Function_52\" name=\"Modified MM\" type=\"UserDefined\" reversible=\"unspecified\">\n"
  "      <Expression>\n"
  "        a_k*y*(x/(x+k))\n"
  "      </Expression>\n"
  "      <ListOfParameterDescriptions>\n"
  "        <ParameterDescription key=\"FunctionParameter_272\" name=\"a_k\" order=\"0\" role=\"constant\"/>\n"
  "        <ParameterDescription key=\"FunctionParameter_250\" name=\"y\" order=\"1\" role=\"modifier\"/>\n"
  "        <ParameterDescription key=\"FunctionParameter_278\" name=\"x\" order=\"2\" role=\"substrate\"/>\n"
  "        <ParameterDescription key=\"FunctionParameter_264\" name=\"k\" order=\"3\" role=\"constant\"/>\n"
  "      </ListOfParameterDescriptions>\n"
  "    </Function>\n"
  "    <Function key=\"Function_67\" name=\"Objective Function 2\" type=\"Expression\">\n"
  "      <Expression>\n"
  "        \n"
  "      </Expression>\n"
  "    </Function>\n"
  "  </ListOfFunctions>\n"
  "  <Model key=\"Model_1\" name=\"New Model\" timeUnit=\"min\" volumeUnit=\"ml\" quantityUnit=\"mmol\" type=\"deterministic\">\n"
  "    <Comment>\n"
  "      <body xmlns=\"http://www.w3.org/1999/xhtml\">\n"
  "        \n"
  "      </body>\n"
  "    </Comment>\n"
  "    <ListOfCompartments>\n"
  "      <Compartment key=\"Compartment_1\" name=\"compartment\" simulationType=\"fixed\">\n"
  "      </Compartment>\n"
  "    </ListOfCompartments>\n"
  "    <ListOfMetabolites>\n"
  "      <Metabolite key=\"Metabolite_5\" name=\"x\" simulationType=\"reactions\" compartment=\"Compartment_1\">\n"
  "        <InitialExpression>\n"
  "          0.66\n"
  "        </InitialExpression>\n"
  "      </Metabolite>\n"
  "      <Metabolite key=\"Metabolite_4\" name=\"y0\" simulationType=\"reactions\" compartment=\"Compartment_1\">\n"
  "      </Metabolite>\n"
  "      <Metabolite key=\"Metabolite_3\" name=\"y1\" simulationType=\"reactions\" compartment=\"Compartment_1\">\n"
  "        <InitialExpression>\n"
  "          0.92\n"
  "        </InitialExpression>\n"
  "      </Metabolite>\n"
  "    </ListOfMetabolites>\n"
  "    <ListOfReactions>\n"
  "      <Reaction key=\"Reaction_15\" name=\"y0Prod\" reversible=\"false\">\n"
  "        <ListOfProducts>\n"
  "          <Product metabolite=\"Metabolite_4\" stoichiometry=\"1\"/>\n"
  "        </ListOfProducts>\n"
  "        <ListOfModifiers>\n"
  "          <Modifier metabolite=\"Metabolite_5\" stoichiometry=\"1\"/>\n"
  "        </ListOfModifiers>\n"
  "        <ListOfConstants>\n"
  "          <Constant key=\"Parameter_290\" name=\"k\" value=\"0.1\"/>\n"
  "        </ListOfConstants>\n"
  "        <KineticLaw function=\"Function_63\">\n"
  "          <ListOfCallParameters>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_297\">\n"
  "              <SourceParameter reference=\"Parameter_290\"/>\n"
  "            </CallParameter>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_268\">\n"
  "              <SourceParameter reference=\"Metabolite_5\"/>\n"
  "            </CallParameter>\n"
  "          </ListOfCallParameters>\n"
  "        </KineticLaw>\n"
  "      </Reaction>\n"
  "      <Reaction key=\"Reaction_14\" name=\"y0toy1\" reversible=\"false\">\n"
  "        <ListOfSubstrates>\n"
  "          <Substrate metabolite=\"Metabolite_4\" stoichiometry=\"1\"/>\n"
  "        </ListOfSubstrates>\n"
  "        <ListOfProducts>\n"
  "          <Product metabolite=\"Metabolite_3\" stoichiometry=\"1\"/>\n"
  "        </ListOfProducts>\n"
  "        <ListOfConstants>\n"
  "          <Constant key=\"Parameter_203\" name=\"k1\" value=\"0.1\"/>\n"
  "        </ListOfConstants>\n"
  "        <KineticLaw function=\"Function_13\">\n"
  "          <ListOfCallParameters>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_81\">\n"
  "              <SourceParameter reference=\"Parameter_203\"/>\n"
  "            </CallParameter>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_79\">\n"
  "              <SourceParameter reference=\"Metabolite_4\"/>\n"
  "            </CallParameter>\n"
  "          </ListOfCallParameters>\n"
  "        </KineticLaw>\n"
  "      </Reaction>\n"
  "      <Reaction key=\"Reaction_12\" name=\"y1Deg\" reversible=\"false\">\n"
  "        <ListOfSubstrates>\n"
  "          <Substrate metabolite=\"Metabolite_3\" stoichiometry=\"1\"/>\n"
  "        </ListOfSubstrates>\n"
  "        <ListOfConstants>\n"
  "          <Constant key=\"Parameter_102\" name=\"k1\" value=\"0.1\"/>\n"
  "        </ListOfConstants>\n"
  "        <KineticLaw function=\"Function_13\">\n"
  "          <ListOfCallParameters>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_81\">\n"
  "              <SourceParameter reference=\"Parameter_102\"/>\n"
  "            </CallParameter>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_79\">\n"
  "              <SourceParameter reference=\"Metabolite_3\"/>\n"
  "            </CallParameter>\n"
  "          </ListOfCallParameters>\n"
  "        </KineticLaw>\n"
  "      </Reaction>\n"
  "      <Reaction key=\"Reaction_11\" name=\"y0Deg\" reversible=\"false\">\n"
  "        <ListOfSubstrates>\n"
  "          <Substrate metabolite=\"Metabolite_4\" stoichiometry=\"1\"/>\n"
  "        </ListOfSubstrates>\n"
  "        <ListOfConstants>\n"
  "          <Constant key=\"Parameter_282\" name=\"k1\" value=\"0.1\"/>\n"
  "        </ListOfConstants>\n"
  "        <KineticLaw function=\"Function_13\">\n"
  "          <ListOfCallParameters>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_81\">\n"
  "              <SourceParameter reference=\"Parameter_282\"/>\n"
  "            </CallParameter>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_79\">\n"
  "              <SourceParameter reference=\"Metabolite_4\"/>\n"
  "            </CallParameter>\n"
  "          </ListOfCallParameters>\n"
  "        </KineticLaw>\n"
  "      </Reaction>\n"
  "      <Reaction key=\"Reaction_8\" name=\"xDegMM\" reversible=\"false\">\n"
  "        <ListOfSubstrates>\n"
  "          <Substrate metabolite=\"Metabolite_5\" stoichiometry=\"1\"/>\n"
  "        </ListOfSubstrates>\n"
  "        <ListOfModifiers>\n"
  "          <Modifier metabolite=\"Metabolite_3\" stoichiometry=\"1\"/>\n"
  "        </ListOfModifiers>\n"
  "        <ListOfConstants>\n"
  "          <Constant key=\"Parameter_412\" name=\"a_k\" value=\"0.1\"/>\n"
  "          <Constant key=\"Parameter_139\" name=\"k\" value=\"0.1\"/>\n"
  "        </ListOfConstants>\n"
  "        <KineticLaw function=\"Function_52\">\n"
  "          <ListOfCallParameters>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_272\">\n"
  "              <SourceParameter reference=\"Parameter_412\"/>\n"
  "            </CallParameter>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_250\">\n"
  "              <SourceParameter reference=\"Metabolite_3\"/>\n"
  "            </CallParameter>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_278\">\n"
  "              <SourceParameter reference=\"Metabolite_5\"/>\n"
  "            </CallParameter>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_264\">\n"
  "              <SourceParameter reference=\"Parameter_139\"/>\n"
  "            </CallParameter>\n"
  "          </ListOfCallParameters>\n"
  "        </KineticLaw>\n"
  "      </Reaction>\n"
  "      <Reaction key=\"Reaction_7\" name=\"y1Prod\" reversible=\"false\">\n"
  "        <ListOfProducts>\n"
  "          <Product metabolite=\"Metabolite_3\" stoichiometry=\"1\"/>\n"
  "        </ListOfProducts>\n"
  "        <ListOfModifiers>\n"
  "          <Modifier metabolite=\"Metabolite_4\" stoichiometry=\"1\"/>\n"
  "        </ListOfModifiers>\n"
  "        <ListOfConstants>\n"
  "          <Constant key=\"Parameter_275\" name=\"k\" value=\"0.1\"/>\n"
  "        </ListOfConstants>\n"
  "        <KineticLaw function=\"Function_63\">\n"
  "          <ListOfCallParameters>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_297\">\n"
  "              <SourceParameter reference=\"Parameter_275\"/>\n"
  "            </CallParameter>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_268\">\n"
  "              <SourceParameter reference=\"Metabolite_4\"/>\n"
  "            </CallParameter>\n"
  "          </ListOfCallParameters>\n"
  "        </KineticLaw>\n"
  "      </Reaction>\n"
  "      <Reaction key=\"Reaction_1\" name=\"xProd\" reversible=\"false\">\n"
  "        <ListOfProducts>\n"
  "          <Product metabolite=\"Metabolite_5\" stoichiometry=\"1\"/>\n"
  "        </ListOfProducts>\n"
  "        <ListOfModifiers>\n"
  "          <Modifier metabolite=\"Metabolite_3\" stoichiometry=\"1\"/>\n"
  "        </ListOfModifiers>\n"
  "        <ListOfConstants>\n"
  "          <Constant key=\"Parameter_339\" name=\"v\" value=\"0.1\"/>\n"
  "        </ListOfConstants>\n"
  "        <KineticLaw function=\"Function_6\">\n"
  "          <ListOfCallParameters>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_49\">\n"
  "              <SourceParameter reference=\"Parameter_339\"/>\n"
  "            </CallParameter>\n"
  "          </ListOfCallParameters>\n"
  "        </KineticLaw>\n"
  "      </Reaction>\n"
  "      <Reaction key=\"Reaction_0\" name=\"xProdLin\" reversible=\"false\">\n"
  "        <ListOfProducts>\n"
  "          <Product metabolite=\"Metabolite_5\" stoichiometry=\"1\"/>\n"
  "        </ListOfProducts>\n"
  "        <ListOfModifiers>\n"
  "          <Modifier metabolite=\"Metabolite_5\" stoichiometry=\"1\"/>\n"
  "        </ListOfModifiers>\n"
  "        <ListOfConstants>\n"
  "          <Constant key=\"Parameter_306\" name=\"k\" value=\"0.1\"/>\n"
  "        </ListOfConstants>\n"
  "        <KineticLaw function=\"Function_63\">\n"
  "          <ListOfCallParameters>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_297\">\n"
  "              <SourceParameter reference=\"Parameter_306\"/>\n"
  "            </CallParameter>\n"
  "            <CallParameter functionParameter=\"FunctionParameter_268\">\n"
  "              <SourceParameter reference=\"Metabolite_5\"/>\n"
  "            </CallParameter>\n"
  "          </ListOfCallParameters>\n"
  "        </KineticLaw>\n"
  "      </Reaction>\n"
  "    </ListOfReactions>\n"
  "    <StateTemplate>\n"
  "      <StateTemplateVariable objectReference=\"Model_1\"/>\n"
  "      <StateTemplateVariable objectReference=\"Metabolite_5\"/>\n"
  "      <StateTemplateVariable objectReference=\"Metabolite_4\"/>\n"
  "      <StateTemplateVariable objectReference=\"Metabolite_3\"/>\n"
  "      <StateTemplateVariable objectReference=\"Compartment_1\"/>\n"
  "    </StateTemplate>\n"
  "    <InitialState type=\"initialState\">\n"
  "      0 3.974613390000001e+20 1.2044283e+20 5.540370180000001e+20 1\n"
  "    </InitialState>\n"
  "  </Model>\n"
  "  <ListOfTasks>\n"
  "    <Task key=\"Task_15\" name=\"Steady-State\" type=\"steadyState\" scheduled=\"false\" updateModel=\"false\">\n"
  "      <Report reference=\"Report_13\" target=\"\" append=\"1\"/>\n"
  "      <Problem>\n"
  "        <Parameter name=\"JacobianRequested\" type=\"bool\" value=\"1\"/>\n"
  "        <Parameter name=\"StabilityAnalysisRequested\" type=\"bool\" value=\"1\"/>\n"
  "      </Problem>\n"
  "      <Method name=\"Enhanced Newton\" type=\"EnhancedNewton\">\n"
  "        <Parameter name=\"Resolution\" type=\"unsignedFloat\" value=\"1e-09\"/>\n"
  "        <Parameter name=\"Derivation Factor\" type=\"unsignedFloat\" value=\"0.001\"/>\n"
  "        <Parameter name=\"Use Newton\" type=\"bool\" value=\"1\"/>\n"
  "        <Parameter name=\"Use Integration\" type=\"bool\" value=\"1\"/>\n"
  "        <Parameter name=\"Use Back Integration\" type=\"bool\" value=\"1\"/>\n"
  "        <Parameter name=\"Accept Negative Concentrations\" type=\"bool\" value=\"0\"/>\n"
  "        <Parameter name=\"Iteration Limit\" type=\"unsignedInteger\" value=\"50\"/>\n"
  "      </Method>\n"
  "    </Task>\n"
  "    <Task key=\"Task_7\" name=\"Time-Course\" type=\"timeCourse\" scheduled=\"false\" updateModel=\"false\">\n"
  "      <Problem>\n"
  "        <Parameter name=\"StepNumber\" type=\"unsignedInteger\" value=\"3500\"/>\n"
  "        <Parameter name=\"StepSize\" type=\"float\" value=\"0.01\"/>\n"
  "        <Parameter name=\"Duration\" type=\"float\" value=\"35\"/>\n"
  "        <Parameter name=\"TimeSeriesRequested\" type=\"bool\" value=\"1\"/>\n"
  "        <Parameter name=\"OutputStartTime\" type=\"float\" value=\"0\"/>\n"
  "      </Problem>\n"
  "      <Method name=\"Deterministic (LSODA)\" type=\"Deterministic(LSODA)\">\n"
  "        <Parameter name=\"Integrate Reduced Model\" type=\"bool\" value=\"0\"/>\n"
  "        <Parameter name=\"Relative Tolerance\" type=\"unsignedFloat\" value=\"1e-06\"/>\n"
  "        <Parameter name=\"Absolute Tolerance\" type=\"unsignedFloat\" value=\"1e-12\"/>\n"
  "        <Parameter name=\"Adams Max Order\" type=\"unsignedInteger\" value=\"12\"/>\n"
  "        <Parameter name=\"BDF Max Order\" type=\"unsignedInteger\" value=\"5\"/>\n"
  "        <Parameter name=\"Max Internal Steps\" type=\"unsignedInteger\" value=\"10000\"/>\n"
  "      </Method>\n"
  "    </Task>\n"
  "    <Task key=\"Task_13\" name=\"Scan\" type=\"scan\" scheduled=\"false\" updateModel=\"false\">\n"
  "      <Problem>\n"
  "        <Parameter name=\"Subtask\" type=\"unsignedInteger\" value=\"1\"/>\n"
  "        <ParameterGroup name=\"ScanItems\">\n"
  "        </ParameterGroup>\n"
  "        <Parameter name=\"Output in subtask\" type=\"bool\" value=\"1\"/>\n"
  "        <Parameter name=\"Adjust initial conditions\" type=\"bool\" value=\"0\"/>\n"
  "      </Problem>\n"
  "      <Method name=\"Scan Framework\" type=\"ScanFramework\">\n"
  "      </Method>\n"
  "    </Task>\n"
  "    <Task key=\"Task_8\" name=\"Elementary Flux Modes\" type=\"fluxMode\" scheduled=\"false\" updateModel=\"false\">\n"
  "      <Report reference=\"Report_12\" target=\"\" append=\"1\"/>\n"
  "      <Problem>\n"
  "      </Problem>\n"
  "      <Method name=\"EFM Algorithm\" type=\"EFMAlgorithm\">\n"
  "      </Method>\n"
  "    </Task>\n"
  "    <Task key=\"Task_11\" name=\"Optimization\" type=\"optimization\" scheduled=\"false\" updateModel=\"false\">\n"
  "      <Report reference=\"Report_11\" target=\"\" append=\"1\"/>\n"
  "      <Problem>\n"
  "        <Parameter name=\"Steady-State\" type=\"key\" value=\"\"/>\n"
  "        <Parameter name=\"Time-Course\" type=\"key\" value=\"Task_7\"/>\n"
  "        <Parameter name=\"ObjectiveFunction\" type=\"key\" value=\"Function_67\"/>\n"
  "        <Parameter name=\"Maximize\" type=\"bool\" value=\"0\"/>\n"
  "        <ParameterGroup name=\"OptimizationItemList\">\n"
  "        </ParameterGroup>\n"
  "        <ParameterGroup name=\"OptimizationConstraintList\">\n"
  "        </ParameterGroup>\n"
  "      </Problem>\n"
  "      <Method name=\"Random Search\" type=\"RandomSearch\">\n"
  "        <Parameter name=\"Number of Iterations\" type=\"unsignedInteger\" value=\"100000\"/>\n"
  "        <Parameter name=\"Random Number Generator\" type=\"unsignedInteger\" value=\"1\"/>\n"
  "        <Parameter name=\"Seed\" type=\"unsignedInteger\" value=\"0\"/>\n"
  "      </Method>\n"
  "    </Task>\n"
  "    <Task key=\"Task_20\" name=\"Parameter Estimation\" type=\"parameterFitting\" scheduled=\"true\" updateModel=\"true\">\n"
  "      <Report reference=\"Report_10\" target=\"\" append=\"1\"/>\n"
  "      <Problem>\n"
  "        <Parameter name=\"Steady-State\" type=\"key\" value=\"Task_15\"/>\n"
  "        <Parameter name=\"Time-Course\" type=\"key\" value=\"Task_7\"/>\n"
  "        <ParameterGroup name=\"OptimizationItemList\">\n"
  "          <ParameterGroup name=\"FitItem\">\n"
  "            <Parameter name=\"ObjectCN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Reactions[y0Prod],ParameterGroup=Parameters,Parameter=k,Reference=Value\"/>\n"
  "            <Parameter name=\"LowerBound\" type=\"cn\" value=\"0.001\"/>\n"
  "            <Parameter name=\"UpperBound\" type=\"cn\" value=\"100\"/>\n"
  "            <Parameter name=\"StartValue\" type=\"float\" value=\"0.1\"/>\n"
  "            <ParameterGroup name=\"Affected Experiments\">\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "          <ParameterGroup name=\"FitItem\">\n"
  "            <Parameter name=\"ObjectCN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Reactions[y0toy1],ParameterGroup=Parameters,Parameter=k1,Reference=Value\"/>\n"
  "            <Parameter name=\"LowerBound\" type=\"cn\" value=\"0.001\"/>\n"
  "            <Parameter name=\"UpperBound\" type=\"cn\" value=\"100\"/>\n"
  "            <Parameter name=\"StartValue\" type=\"float\" value=\"0.1\"/>\n"
  "            <ParameterGroup name=\"Affected Experiments\">\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "          <ParameterGroup name=\"FitItem\">\n"
  "            <Parameter name=\"ObjectCN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Reactions[y1Deg],ParameterGroup=Parameters,Parameter=k1,Reference=Value\"/>\n"
  "            <Parameter name=\"LowerBound\" type=\"cn\" value=\"0.001\"/>\n"
  "            <Parameter name=\"UpperBound\" type=\"cn\" value=\"100\"/>\n"
  "            <Parameter name=\"StartValue\" type=\"float\" value=\"0.1\"/>\n"
  "            <ParameterGroup name=\"Affected Experiments\">\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "          <ParameterGroup name=\"FitItem\">\n"
  "            <Parameter name=\"ObjectCN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Reactions[y0Deg],ParameterGroup=Parameters,Parameter=k1,Reference=Value\"/>\n"
  "            <Parameter name=\"LowerBound\" type=\"cn\" value=\"0.001\"/>\n"
  "            <Parameter name=\"UpperBound\" type=\"cn\" value=\"100\"/>\n"
  "            <Parameter name=\"StartValue\" type=\"float\" value=\"0.1\"/>\n"
  "            <ParameterGroup name=\"Affected Experiments\">\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "          <ParameterGroup name=\"FitItem\">\n"
  "            <Parameter name=\"ObjectCN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Reactions[xDegMM],ParameterGroup=Parameters,Parameter=a_k,Reference=Value\"/>\n"
  "            <Parameter name=\"LowerBound\" type=\"cn\" value=\"0.001\"/>\n"
  "            <Parameter name=\"UpperBound\" type=\"cn\" value=\"100\"/>\n"
  "            <Parameter name=\"StartValue\" type=\"float\" value=\"0.1\"/>\n"
  "            <ParameterGroup name=\"Affected Experiments\">\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "          <ParameterGroup name=\"FitItem\">\n"
  "            <Parameter name=\"ObjectCN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Reactions[xDegMM],ParameterGroup=Parameters,Parameter=k,Reference=Value\"/>\n"
  "            <Parameter name=\"LowerBound\" type=\"cn\" value=\"0.001\"/>\n"
  "            <Parameter name=\"UpperBound\" type=\"cn\" value=\"100\"/>\n"
  "            <Parameter name=\"StartValue\" type=\"float\" value=\"0.1\"/>\n"
  "            <ParameterGroup name=\"Affected Experiments\">\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "          <ParameterGroup name=\"FitItem\">\n"
  "            <Parameter name=\"ObjectCN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Reactions[y1Prod],ParameterGroup=Parameters,Parameter=k,Reference=Value\"/>\n"
  "            <Parameter name=\"LowerBound\" type=\"cn\" value=\"0.001\"/>\n"
  "            <Parameter name=\"UpperBound\" type=\"cn\" value=\"100\"/>\n"
  "            <Parameter name=\"StartValue\" type=\"float\" value=\"0.1\"/>\n"
  "            <ParameterGroup name=\"Affected Experiments\">\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "          <ParameterGroup name=\"FitItem\">\n"
  "            <Parameter name=\"ObjectCN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Reactions[xProd],ParameterGroup=Parameters,Parameter=k,Reference=Value\"/>\n"
  "            <Parameter name=\"LowerBound\" type=\"cn\" value=\"0.001\"/>\n"
  "            <Parameter name=\"UpperBound\" type=\"cn\" value=\"100\"/>\n"
  "            <Parameter name=\"StartValue\" type=\"float\" value=\"0.1\"/>\n"
  "            <ParameterGroup name=\"Affected Experiments\">\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "          <ParameterGroup name=\"FitItem\">\n"
  "            <Parameter name=\"ObjectCN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Compartments[compartment],Vector=Metabolites[y0],Reference=InitialConcentration\"/>\n"
  "            <Parameter name=\"LowerBound\" type=\"cn\" value=\"0.01\"/>\n"
  "            <Parameter name=\"UpperBound\" type=\"cn\" value=\"1\"/>\n"
  "            <Parameter name=\"StartValue\" type=\"float\" value=\"0.2\"/>\n"
  "            <ParameterGroup name=\"Affected Experiments\">\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "        </ParameterGroup>\n"
  "        <ParameterGroup name=\"OptimizationConstraintList\">\n"
  "        </ParameterGroup>\n"
  "        <ParameterGroup name=\"Experiment Set\">\n"
  "          <ParameterGroup name=\"Experiment\">\n"
  "            <Parameter name=\"Key\" type=\"key\" value=\"Experiment_2\"/>\n"
  "            <Parameter name=\"File Name\" type=\"file\" value=\"Geva-ZatorskyCutoff.txt\"/>\n"
  "            <Parameter name=\"First Row\" type=\"unsignedInteger\" value=\"1\"/>\n"
  "            <Parameter name=\"Last Row\" type=\"unsignedInteger\" value=\"93\"/>\n"
  "            <Parameter name=\"Experiment Type\" type=\"unsignedInteger\" value=\"1\"/>\n"
  "            <Parameter name=\"Separator\" type=\"string\" value=\"&#x09;\"/>\n"
  "            <Parameter name=\"Weight Method\" type=\"unsignedInteger\" value=\"2\"/>\n"
  "            <Parameter name=\"Data is Row Oriented\" type=\"bool\" value=\"1\"/>\n"
  "            <Parameter name=\"Row containing Names\" type=\"unsignedInteger\" value=\"1\"/>\n"
  "            <Parameter name=\"Number of Columns\" type=\"unsignedInteger\" value=\"3\"/>\n"
  "            <ParameterGroup name=\"Object Map\">\n"
  "              <ParameterGroup name=\"0\">\n"
  "                <Parameter name=\"Role\" type=\"unsignedInteger\" value=\"3\"/>\n"
  "              </ParameterGroup>\n"
  "              <ParameterGroup name=\"1\">\n"
  "                <Parameter name=\"Role\" type=\"unsignedInteger\" value=\"2\"/>\n"
  "                <Parameter name=\"Object CN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Compartments[compartment],Vector=Metabolites[x],Reference=Concentration\"/>\n"
  "              </ParameterGroup>\n"
  "              <ParameterGroup name=\"2\">\n"
  "                <Parameter name=\"Role\" type=\"unsignedInteger\" value=\"2\"/>\n"
  "                <Parameter name=\"Object CN\" type=\"cn\" value=\"CN=Root,Model=New Model,Vector=Compartments[compartment],Vector=Metabolites[y1],Reference=Concentration\"/>\n"
  "              </ParameterGroup>\n"
  "            </ParameterGroup>\n"
  "          </ParameterGroup>\n"
  "        </ParameterGroup>\n"
  "      </Problem>\n"
  "      <Method name=\"Evolutionary Programming\" type=\"EvolutionaryProgram\">\n"
  "        <Parameter name=\"Number of Generations\" type=\"unsignedInteger\" value=\"2000\"/>\n"
  "        <Parameter name=\"Population Size\" type=\"unsignedInteger\" value=\"100\"/>\n"
  "        <Parameter name=\"Random Number Generator\" type=\"unsignedInteger\" value=\"1\"/>\n"
  "        <Parameter name=\"Seed\" type=\"unsignedInteger\" value=\"0\"/>\n"
  "      </Method>\n"
  "    </Task>\n"
  "    <Task key=\"Task_19\" name=\"Metabolic Control Analysis\" type=\"metabolicControlAnalysis\" scheduled=\"false\" updateModel=\"false\">\n"
  "      <Report reference=\"Report_9\" target=\"\" append=\"1\"/>\n"
  "      <Problem>\n"
  "        <Parameter name=\"Steady-State\" type=\"key\" value=\"Task_15\"/>\n"
  "      </Problem>\n"
  "      <Method name=\"MCA Method (Reder)\" type=\"MCAMethod(Reder)\">\n"
  "        <Parameter name=\"Modulation Factor\" type=\"unsignedFloat\" value=\"1e-09\"/>\n"
  "      </Method>\n"
  "    </Task>\n"
  "    <Task key=\"Task_18\" name=\"Lyapunov Exponents\" type=\"lyapunovExponents\" scheduled=\"false\" updateModel=\"false\">\n"
  "      <Report reference=\"Report_8\" target=\"\" append=\"1\"/>\n"
  "      <Problem>\n"
  "        <Parameter name=\"ExponentNumber\" type=\"unsignedInteger\" value=\"3\"/>\n"
  "        <Parameter name=\"DivergenceRequested\" type=\"bool\" value=\"1\"/>\n"
  "        <Parameter name=\"TransientTime\" type=\"float\" value=\"0\"/>\n"
  "      </Problem>\n"
  "      <Method name=\"Wolf Method\" type=\"WolfMethod\">\n"
  "        <Parameter name=\"Orthonormalization Interval\" type=\"unsignedFloat\" value=\"1\"/>\n"
  "        <Parameter name=\"Overall time\" type=\"unsignedFloat\" value=\"1000\"/>\n"
  "        <Parameter name=\"Relative Tolerance\" type=\"unsignedFloat\" value=\"1e-06\"/>\n"
  "        <Parameter name=\"Use Default Absolute Tolerance\" type=\"bool\" value=\"1\"/>\n"
  "        <Parameter name=\"Absolute Tolerance\" type=\"unsignedFloat\" value=\"1e-12\"/>\n"
  "        <Parameter name=\"Adams Max Order\" type=\"unsignedInteger\" value=\"12\"/>\n"
  "        <Parameter name=\"BDF Max Order\" type=\"unsignedInteger\" value=\"5\"/>\n"
  "        <Parameter name=\"Max Internal Steps\" type=\"unsignedInteger\" value=\"10000\"/>\n"
  "      </Method>\n"
  "    </Task>\n"
  "    <Task key=\"Task_17\" name=\"Sensitivities\" type=\"sensitivities\" scheduled=\"false\" updateModel=\"false\">\n"
  "      <Report reference=\"Report_7\" target=\"\" append=\"1\"/>\n"
  "      <Problem>\n"
  "        <Parameter name=\"SubtaskType\" type=\"unsignedInteger\" value=\"1\"/>\n"
  "        <ParameterGroup name=\"TargetFunctions\">\n"
  "          <Parameter name=\"SingleObject\" type=\"cn\" value=\"\"/>\n"
  "          <Parameter name=\"ObjectListType\" type=\"unsignedInteger\" value=\"7\"/>\n"
  "        </ParameterGroup>\n"
  "        <ParameterGroup name=\"ListOfVariables\">\n"
  "          <ParameterGroup name=\"Variables\">\n"
  "            <Parameter name=\"SingleObject\" type=\"cn\" value=\"\"/>\n"
  "            <Parameter name=\"ObjectListType\" type=\"unsignedInteger\" value=\"41\"/>\n"
  "          </ParameterGroup>\n"
  "        </ParameterGroup>\n"
  "      </Problem>\n"
  "      <Method name=\"Sensitivities Method\" type=\"SensitivitiesMethod\">\n"
  "        <Parameter name=\"Delta factor\" type=\"unsignedFloat\" value=\"1e-06\"/>\n"
  "        <Parameter name=\"Delta minimum\" type=\"unsignedFloat\" value=\"1e-12\"/>\n"
  "      </Method>\n"
  "    </Task>\n"
  "    <Task key=\"Task_10\" name=\"Moieties\" type=\"moieties\" scheduled=\"false\" updateModel=\"false\">\n"
  "      <Problem>\n"
  "      </Problem>\n"
  "      <Method name=\"Householder Reduction\" type=\"Householder\">\n"
  "      </Method>\n"
  "    </Task>\n"
  "  </ListOfTasks>\n"
  "  <ListOfReports>\n"
  "    <Report key=\"Report_13\" name=\"Steady-State\" taskType=\"steadyState\" separator=\"&#x09;\" precision=\"6\">\n"
  "      <Comment>\n"
  "        <body xmlns=\"http://www.w3.org/1999/xhtml\">\n"
  "          Automatically generated report.\n"
  "        </body>\n"
  "      </Comment>\n"
  "      <Footer>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Steady-State]\"/>\n"
  "      </Footer>\n"
  "    </Report>\n"
  "    <Report key=\"Report_12\" name=\"Elementary Flux Modes\" taskType=\"fluxMode\" separator=\"&#x09;\" precision=\"6\">\n"
  "      <Comment>\n"
  "        <body xmlns=\"http://www.w3.org/1999/xhtml\">\n"
  "          Automatically generated report.\n"
  "        </body>\n"
  "      </Comment>\n"
  "      <Footer>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Elementary Flux Modes],Object=Result\"/>\n"
  "      </Footer>\n"
  "    </Report>\n"
  "    <Report key=\"Report_11\" name=\"Optimization\" taskType=\"optimization\" separator=\"&#x09;\" precision=\"6\">\n"
  "      <Comment>\n"
  "        <body xmlns=\"http://www.w3.org/1999/xhtml\">\n"
  "          Automatically generated report.\n"
  "        </body>\n"
  "      </Comment>\n"
  "      <Header>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Optimization],Object=Description\"/>\n"
  "        <Object cn=\"String=\\[Function Evaluations\\]\"/>\n"
  "        <Object cn=\"Separator=&#x09;\"/>\n"
  "        <Object cn=\"String=\\[Best Value\\]\"/>\n"
  "        <Object cn=\"Separator=&#x09;\"/>\n"
  "        <Object cn=\"String=\\[Best Parameters\\]\"/>\n"
  "      </Header>\n"
  "      <Body>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Optimization],Problem=Optimization,Reference=Function Evaluations\"/>\n"
  "        <Object cn=\"Separator=&#x09;\"/>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Optimization],Problem=Optimization,Reference=Best Value\"/>\n"
  "        <Object cn=\"Separator=&#x09;\"/>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Optimization],Problem=Optimization,Reference=Best Parameters\"/>\n"
  "      </Body>\n"
  "      <Footer>\n"
  "        <Object cn=\"String=&#x0a;\"/>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Optimization],Object=Result\"/>\n"
  "      </Footer>\n"
  "    </Report>\n"
  "    <Report key=\"Report_10\" name=\"Parameter Estimation\" taskType=\"parameterFitting\" separator=\"&#x09;\" precision=\"6\">\n"
  "      <Comment>\n"
  "        <body xmlns=\"http://www.w3.org/1999/xhtml\">\n"
  "          Automatically generated report.\n"
  "        </body>\n"
  "      </Comment>\n"
  "      <Header>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Object=Description\"/>\n"
  "        <Object cn=\"String=\\[Function Evaluations\\]\"/>\n"
  "        <Object cn=\"Separator=&#x09;\"/>\n"
  "        <Object cn=\"String=\\[Best Value\\]\"/>\n"
  "        <Object cn=\"Separator=&#x09;\"/>\n"
  "        <Object cn=\"String=\\[Best Parameters\\]\"/>\n"
  "      </Header>\n"
  "      <Body>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,Reference=Function Evaluations\"/>\n"
  "        <Object cn=\"Separator=&#x09;\"/>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,Reference=Best Value\"/>\n"
  "        <Object cn=\"Separator=&#x09;\"/>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,Reference=Best Parameters\"/>\n"
  "      </Body>\n"
  "      <Footer>\n"
  "        <Object cn=\"String=&#x0a;\"/>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Object=Result\"/>\n"
  "      </Footer>\n"
  "    </Report>\n"
  "    <Report key=\"Report_9\" name=\"Metabolic Control Analysis\" taskType=\"metabolicControlAnalysis\" separator=\"&#x09;\" precision=\"6\">\n"
  "      <Comment>\n"
  "        <body xmlns=\"http://www.w3.org/1999/xhtml\">\n"
  "          Automatically generated report.\n"
  "        </body>\n"
  "      </Comment>\n"
  "      <Header>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Metabolic Control Analysis],Object=Description\"/>\n"
  "      </Header>\n"
  "      <Footer>\n"
  "        <Object cn=\"String=&#x0a;\"/>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Metabolic Control Analysis],Object=Result\"/>\n"
  "      </Footer>\n"
  "    </Report>\n"
  "    <Report key=\"Report_8\" name=\"Lyapunov Exponents\" taskType=\"lyapunovExponents\" separator=\"&#x09;\" precision=\"6\">\n"
  "      <Comment>\n"
  "        <body xmlns=\"http://www.w3.org/1999/xhtml\">\n"
  "          Automatically generated report.\n"
  "        </body>\n"
  "      </Comment>\n"
  "      <Header>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Lyapunov Exponents],Object=Description\"/>\n"
  "      </Header>\n"
  "      <Footer>\n"
  "        <Object cn=\"String=&#x0a;\"/>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Lyapunov Exponents],Object=Result\"/>\n"
  "      </Footer>\n"
  "    </Report>\n"
  "    <Report key=\"Report_7\" name=\"Sensitivities\" taskType=\"sensitivities\" separator=\"&#x09;\" precision=\"6\">\n"
  "      <Comment>\n"
  "        <body xmlns=\"http://www.w3.org/1999/xhtml\">\n"
  "          Automatically generated report.\n"
  "        </body>\n"
  "      </Comment>\n"
  "      <Header>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Sensitivities],Object=Description\"/>\n"
  "      </Header>\n"
  "      <Footer>\n"
  "        <Object cn=\"String=&#x0a;\"/>\n"
  "        <Object cn=\"CN=Root,Vector=TaskList[Sensitivities],Object=Result\"/>\n"
  "      </Footer>\n"
  "    </Report>\n"
  "  </ListOfReports>\n"
  "  <ListOfPlots>\n"
  "    <PlotSpecification name=\"Concentrations, Volumes, and Global Quantity Values\" type=\"Plot2D\" active=\"1\">\n"
  "      <Parameter name=\"log X\" type=\"bool\" value=\"0\"/>\n"
  "      <Parameter name=\"log Y\" type=\"bool\" value=\"0\"/>\n"
  "      <ListOfPlotItems>\n"
  "        <PlotItem name=\"[x]\" type=\"Curve2D\">\n"
  "          <Parameter name=\"Line type\" type=\"unsignedInteger\" value=\"0\"/>\n"
  "          <Parameter name=\"Recording Activity\" type=\"string\" value=\"during\"/>\n"
  "          <ListOfChannels>\n"
  "            <ChannelSpec cn=\"CN=Root,Model=New Model,Reference=Time\"/>\n"
  "            <ChannelSpec cn=\"CN=Root,Model=New Model,Vector=Compartments[compartment],Vector=Metabolites[x],Reference=Concentration\"/>\n"
  "          </ListOfChannels>\n"
  "        </PlotItem>\n"
  "        <PlotItem name=\"[y0]\" type=\"Curve2D\">\n"
  "          <Parameter name=\"Line type\" type=\"unsignedInteger\" value=\"0\"/>\n"
  "          <Parameter name=\"Recording Activity\" type=\"string\" value=\"during\"/>\n"
  "          <ListOfChannels>\n"
  "            <ChannelSpec cn=\"CN=Root,Model=New Model,Reference=Time\"/>\n"
  "            <ChannelSpec cn=\"CN=Root,Model=New Model,Vector=Compartments[compartment],Vector=Metabolites[y0],Reference=Concentration\"/>\n"
  "          </ListOfChannels>\n"
  "        </PlotItem>\n"
  "        <PlotItem name=\"[y]\" type=\"Curve2D\">\n"
  "          <Parameter name=\"Line type\" type=\"unsignedInteger\" value=\"0\"/>\n"
  "          <Parameter name=\"Recording Activity\" type=\"string\" value=\"during\"/>\n"
  "          <ListOfChannels>\n"
  "            <ChannelSpec cn=\"CN=Root,Model=New Model,Reference=Time\"/>\n"
  "            <ChannelSpec cn=\"CN=Root,Model=New Model,Vector=Compartments[compartment],Vector=Metabolites[y1],Reference=Concentration\"/>\n"
  "          </ListOfChannels>\n"
  "        </PlotItem>\n"
  "      </ListOfPlotItems>\n"
  "    </PlotSpecification>\n"
  "    <PlotSpecification name=\"Experiment\" type=\"Plot2D\" active=\"1\">\n"
  "      <Parameter name=\"log X\" type=\"bool\" value=\"0\"/>\n"
  "      <Parameter name=\"log Y\" type=\"bool\" value=\"0\"/>\n"
  "      <ListOfPlotItems>\n"
  "        <PlotItem name=\"[x](Measured Value)\" type=\"Curve2D\">\n"
  "          <Parameter name=\"Line type\" type=\"unsignedInteger\" value=\"2\"/>\n"
  "          <Parameter name=\"Recording Activity\" type=\"string\" value=\"after\"/>\n"
  "          <ListOfChannels>\n"
  "            <ChannelSpec cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,ParameterGroup=Experiment Set,ParameterGroup=Experiment,Vector=Fitted Points[0],Reference=Independent Value\"/>\n"
  "            <ChannelSpec cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,ParameterGroup=Experiment Set,ParameterGroup=Experiment,Vector=Fitted Points[0],Reference=Measured Value\"/>\n"
  "          </ListOfChannels>\n"
  "        </PlotItem>\n"
  "        <PlotItem name=\"[x](Fitted Value)\" type=\"Curve2D\">\n"
  "          <Parameter name=\"Line type\" type=\"unsignedInteger\" value=\"0\"/>\n"
  "          <Parameter name=\"Recording Activity\" type=\"string\" value=\"after\"/>\n"
  "          <ListOfChannels>\n"
  "            <ChannelSpec cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,ParameterGroup=Experiment Set,ParameterGroup=Experiment,Vector=Fitted Points[0],Reference=Independent Value\"/>\n"
  "            <ChannelSpec cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,ParameterGroup=Experiment Set,ParameterGroup=Experiment,Vector=Fitted Points[0],Reference=Fitted Value\"/>\n"
  "          </ListOfChannels>\n"
  "        </PlotItem>\n"
  "        <PlotItem name=\"[y](Measured Value)\" type=\"Curve2D\">\n"
  "          <Parameter name=\"Line type\" type=\"unsignedInteger\" value=\"2\"/>\n"
  "          <Parameter name=\"Recording Activity\" type=\"string\" value=\"after\"/>\n"
  "          <ListOfChannels>\n"
  "            <ChannelSpec cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,ParameterGroup=Experiment Set,ParameterGroup=Experiment,Vector=Fitted Points[0],Reference=Independent Value\"/>\n"
  "            <ChannelSpec cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,ParameterGroup=Experiment Set,ParameterGroup=Experiment,Vector=Fitted Points[1],Reference=Measured Value\"/>\n"
  "          </ListOfChannels>\n"
  "        </PlotItem>\n"
  "        <PlotItem name=\"[y](Fitted Value)\" type=\"Curve2D\">\n"
  "          <Parameter name=\"Line type\" type=\"unsignedInteger\" value=\"0\"/>\n"
  "          <Parameter name=\"Recording Activity\" type=\"string\" value=\"after\"/>\n"
  "          <ListOfChannels>\n"
  "            <ChannelSpec cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,ParameterGroup=Experiment Set,ParameterGroup=Experiment,Vector=Fitted Points[0],Reference=Independent Value\"/>\n"
  "            <ChannelSpec cn=\"CN=Root,Vector=TaskList[Parameter Estimation],Problem=Parameter Estimation,ParameterGroup=Experiment Set,ParameterGroup=Experiment,Vector=Fitted Points[1],Reference=Fitted Value\"/>\n"
  "          </ListOfChannels>\n"
  "        </PlotItem>\n"
  "      </ListOfPlotItems>\n"
  "    </PlotSpecification>\n"
  "  </ListOfPlots>\n"
  "  <GUI>\n"
  "  </GUI>\n"
  "  <SBMLReference file=\"MasterModel.xml\">\n"
  "    <SBMLMap SBMLid=\"compartment_1\" COPASIkey=\"Compartment_1\"/>\n"
  "    <SBMLMap SBMLid=\"function_1\" COPASIkey=\"Function_52\"/>\n"
  "    <SBMLMap SBMLid=\"function_2\" COPASIkey=\"Function_63\"/>\n"
  "    <SBMLMap SBMLid=\"reaction_3\" COPASIkey=\"Reaction_15\"/>\n"
  "    <SBMLMap SBMLid=\"reaction_4\" COPASIkey=\"Reaction_14\"/>\n"
  "    <SBMLMap SBMLid=\"reaction_5\" COPASIkey=\"Reaction_12\"/>\n"
  "    <SBMLMap SBMLid=\"reaction_8\" COPASIkey=\"Reaction_11\"/>\n"
  "    <SBMLMap SBMLid=\"species_1\" COPASIkey=\"Metabolite_5\"/>\n"
  "    <SBMLMap SBMLid=\"species_2\" COPASIkey=\"Metabolite_4\"/>\n"
  "    <SBMLMap SBMLid=\"species_3\" COPASIkey=\"Metabolite_3\"/>\n"
  "  </SBMLReference>\n"
  "</COPASI>\n"
};
